#!/usr/bin/env python

# Rnmap server v. 0.4.1 Beta
#
# Copyright (C) 2000 Tuomo Makinen (tmakinen@pp.htv.fi)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""
Rnmap server v. 0.4.1 Beta

Copyright (C) 2000 Tuomo Makinen
Redistributable under the terms of the GNU General Public License

Usage:
    -p: port
    -n: Nmap path
    -h: hostname

    --help          - prints this message
    --version       - show version of rnmap server
    --nofilter      - turns filtering rules off
"""

from socket import *
import os
import string
import sys
import thread
import tempfile
import time
import getopt
import signal

CRLF = "\r\n"
SUCCESFUL  = "201:" #Succesful
AUTHPERM   = "202:Access permitted"
AUTHDENI   = "401:Access denied"
SCANFAIL   = "402:Operation failed"
UNKNOWNERR = "403:Unknown command"
BADCHARS   = "404:Operation denied"

# NOTHESECHARS is tuple which has characters that can cause
# unwanted action, and we don't want pass them to system.
NOTHESECHARS = ("|", ";", "<", ">", "&")
                     
Server = ''
Port = 3500
Nmapl = "/usr/bin/nmap"
Filteruse = 1

try:
    opts, args = getopt.getopt(sys.argv[1:], 'p:n:h:', ['help', 'nofilter', 'version'])
except:
    sys.stderr.write("Try 'rnmap_server.py --help' for more information.\n")
    sys.exit()

for o, a in opts:
    if o == '-h': Server = a
    if o == '-p': Port = string.atoi(a)
    if o == '-n': Nmapl = a
    if o == '--help':
        print __doc__
        sys.exit()
    if o == '--version':
        print "Rnmap server v. 0.4.1 Beta"
        sys.exit()
    if o == '--nofilter':
        Filteruse = None
        Fmethod = None
    
#==========================================================================
# read configuration files ...
#==========================================================================
# filter.conf is optional file, if omitted we're not using filtering rules.
def confreader():
    global Filteruse
    if Filteruse == 1:
        global Fmethod
        try:
            # Try to open filtering configuration
            fp = open("filter.conf")
            tmplist = fp.readlines()
            fp.close()
            global Clist
            Clist = []
            for i in xrange(len(tmplist)):
                if "#" in tmplist[i]:
                    continue
                elif tmplist[i][0:] == '\012':
                    continue
                elif tmplist[i][0] == "!":
                    Fmethod = tmplist[i][9:16]
                else:
                    Clist.append(tmplist[i])
            for j in xrange(len(Clist)):
                Clist[j] = Clist[j][:-1]
        except:
            # No filtering rules used
            Filteruse = None
            Fmethod = None
    # userlist file
    try:
        fp = open("users.list")
        ul = fp.readlines()
        global Userlist
        Userlist = {}
        for i in xrange(len(ul)):
            stopsign = 0
            for j in xrange(len(ul[i])): 
                if ul[i][0] == "#":
                    continue
                elif ul[i][0] == " ":
                    continue
                elif ul[i][j] == ":":
                    if stopsign == 0:
                        Userlist[ul[i][:j]] = ul[i][j+1:-1]
                        stopsign = 1
    except:
        logwriter("ulist_failed")
        sys.exit()
#==========================================================================

def waitconnection():
    s = socket(AF_INET, SOCK_STREAM)
    s.bind((Server, Port))
    s.listen(1)
    while 1:
        try:
            conn, addr = s.accept()
        except error:
            pass
        else:
            thread.start_new_thread(newconnection,(s, conn, addr[0]))
            
def newconnection(s, conn, host):
    data = conn.recv(4096)
    if Filteruse != None:
        check = filter(host)
        if check == 0:
            message = AUTHDENI
            logwriter("banned_host", host)
        elif check != 0:
            message = ploadexam(data, host)
        conn.send(message)
        conn.close()
    else:
        message = ploadexam(data, host)
        conn.send(message)
        conn.close()
         
    thread.exit()

def ploadexam(message, host):
    message = cleaner(message)
    cmd_point = string.rfind(message, ":")
    usr_point = string.find(message, ":")
    login = message[:usr_point]
    cmd = message[cmd_point+1:]
    passwd = message[usr_point+1:cmd_point]
    if (cmd_point != -1 and usr_point != -1) and (cmd_point != usr_point):
        if cmd == '':
            auth = authenticate(login, passwd)                        
            if auth == "true":
                logwriter("user_authorized", host, login)
                info = AUTHPERM
            elif auth == "false":
                logwriter("bad_password", host, login)
                info = AUTHDENI
            elif auth == "nouser":
                logwriter("unknown_user", host, login)
                info = AUTHDENI
        elif cmd != '':
            auth = authenticate(login, passwd)
            if auth == "true":
                # Check chars
                for i in xrange(len(cmd)):
                    if cmd[i] in NOTHESECHARS:
                        scanstat = 0
                        break
                    else:
                        scanstat = 1
                if scanstat == 0:
                    logwriter("illegal_parameters", host, login, cmd)
                    info = BADCHARS   
                elif scanstat == 1:
                    info = scanner(login, host, cmd)
            elif auth != "true":
                logwriter("no_permission", host, login)
                info = AUTHDENI
    else:
        info = UNKNOWNERR
        
    return info

def scanner(login, host, cmd):
    tempfile.tempdir = "./tmp"
    tmpresult = tempfile.mktemp()
    syscmd = Nmapl+" "+cmd+" "+tmpresult+" >/dev/null 2>&1"
    scn = os.system(syscmd)
    if scn != 0:
        logwriter("failed_scan", host, login, cmd)
        info = SCANFAIL
    else:
        try:
            fp = open(tmpresult, "r")
        except IOError:
            logwriter("failed_scan", host, login, cmd)
            info = SCANFAIL
        else:
            logwriter("succesful_scan", host, login, cmd)
            info = SUCCESFUL+fp.read()
            fp.close()
            os.unlink(tmpresult)
            
    return info

def authenticate(login, passwd):        
    if Userlist.has_key(login):
        if Userlist[login] == passwd:
            authentication = "true"
        else:
            authentication = "false"
    else:
        authentication = "nouser"

    return (authentication)

def logwriter(information, host=None, login=None, cmd=None):
    thread_pid = os.getpid()
    if information ==  "user_authorized":
        wlog = "["+time.ctime(time.time())+"][%d] user %s from %s authorized.\n"\
               % (thread_pid, login, host)
    elif information == "succesful_scan":
        wlog = "["+time.ctime(time.time())+"][%d] user %s from %s peformed scan: %s\n"\
               % (thread_pid, login, host, cmd)
    elif information ==  "bad_password":
        wlog = "["+time.ctime(time.time())+"][%d] user %s from %s tried to login with wrong password.\n"\
               % (thread_pid, login, host)
    elif information == "unknown_user":
        wlog = "["+time.ctime(time.time())+"][%d] unknown user %s from %s tried to connect.\n"\
               % (thread_pid, login, host)
    elif information ==  "no_permission":
        wlog = "["+time.ctime(time.time())+"][%d] unknown user %s from %s tried to execute scan without permission.\n"\
               % (thread_pid, login, host)
    elif information == "failed_scan":
        wlog = "["+time.ctime(time.time())+"][%d] user %s from %s peformed failed scan: %s\n"\
               % (thread_pid, login, host, cmd)
    elif information == "banned_host":
        wlog = "["+time.ctime(time.time())+"][%d] banned host: %s dropped.\n"\
               % (thread_pid, host)
    elif information ==  "illegal_parameters":
        wlog = "["+time.ctime(time.time())+"][%d] user %s from %s used illegal parameters: %s\n"\
               % (thread_pid, login, host, cmd)
    elif information == "startup":
        wlog = "["+time.ctime(time.time())+"][%d] rnmap server started.\n"\
               % (thread_pid)
    elif information == "sigterm":
        wlog = "["+time.ctime(time.time())+"][%d] received the SIGTERM signal.\n"\
               % (thread_pid)
    elif information == "sighup":
        wlog = "["+time.ctime(time.time())+"][%d] received the SIGHUP signal.\n"\
               % (thread_pid)
    elif information == "ulist_failed":
        wlog = "["+time.ctime(time.time())+"][%d] error reading users.list - server startup failed.\n"\
               % (thread_pid)
    #---    
    try:
        fp = open("rnmap_server.log", "a")
        fp.write(wlog)
        fp.close()
    except IOError:
        wlog = "rnmap_serv: "+time.ctime(time.time())+" Can't write to disk!\n"
        sys.stderr.write(wlog)
        
def cleaner(message):
    if len(message) >= 2:
        if message[-2:] == CRLF:
            message = message[:-2]
        elif message[-1:] in CRLF:
            message = message[:-1]
    
    return message

def filter(host):
    check = None
    hcheck = host
    if Fmethod == "drpexcp":
        if hcheck in Clist:
            check = 1
        else:
            check = 0
    if Fmethod == "allexcp":
        if hcheck in Clist:
            check = 0
        else:
            check = 1
            
    return check

def signal_handler(signum, frame):
    if signum == 1:
        logwriter("sighup")
        confreader()
    elif signum == 15:
        logwriter("sigterm")
        sys.exit()

logwriter("startup")   
# Fork to background ...
pid = os.fork()
if pid != 0:
    sys.exit()
else:
    signal.signal(signal.SIGTERM, signal_handler)
    signal.signal(signal.SIGHUP, signal_handler)
    confreader()
    waitconnection()
